==> ircd-hybrid-ru-7.0.tar.gz: ircd-hybrid-ru/src/translate.c <==
/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  translate.c: 
 *
 *  Copyright (C) 2002 by Elias I. Shtift. ishtift@tagil.svrw.ru
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: $
 */

#include "common.h"
#include "stdinc.h"
#include "fileio.h"
#include "memory.h"
#include "client.h"
#include "translate.h"
#include "s_user.h"
#include "s_log.h"
#include "s_conf.h"

struct Codepage *codepages = NULL;

struct Codepage *translate_codepage_add(const char *charset, const char *file)
{
  struct Codepage *cp;
  int fd = NULL, cnt = NULL;

  if (charset == NULL)
    return NULL;

  if ((cp = translate_codepage_find_byname(charset)))
  {
    return cp;
  }

  if (codepages)
  {
    cp = codepages;
    while (cp->next)
    {
      cp = cp->next;
    }
    cp->next = (struct Codepage*) MyMalloc(sizeof(struct Codepage));
    cp->next->prev = cp;
    cp->next->next = NULL;
    cp = cp->next;
  }
  else
  {
    codepages = cp = (struct Codepage*) MyMalloc(sizeof(struct Codepage));
    cp->prev = cp->next = NULL;
  }

  strlcpy(cp->name, charset, 20);

  if ((fd = file_open(file, O_RDONLY, 0)) == -1)
  {
    if (cp->prev)
      cp->prev->next = NULL;
    MyFree(cp);
    return NULL;
  }
  cnt += read(fd, cp->to, 256);
  cnt += read(fd, cp->from, 256);
  file_close(fd);
  if (cnt == 512)
  {
    return cp;
  }
  else
  {
    if (cp->prev)
      cp->prev->next = NULL;
    MyFree(cp);
    return NULL;
  }
}

struct Codepage *translate_codepage_find_byname(const char *charset)
{
  struct Codepage *cp = codepages;

  while (cp)
  {
    if (!strcasecmp(cp->name, charset))
      break;

    cp = cp->next;
  }
  
  return cp;
}

struct Codepage *translate_change_codepage(struct Client *target_p, struct Codepage *cp)
{
  if (target_p->localClient == NULL)
    return NULL;
  return target_p->localClient->cp = cp;
}

==> ircd-hybrid-ru-7.0.3.tar.gz: ircd-hybrid-ru/src/translate.c <==
/*
 *  ircd-hybrid: an advanced Internet Relay Chat Daemon(ircd).
 *  translate.c: 
 *
 *  Copyright (C) 2002 by Elias I. Shtift. ishtift@tagil.svrw.ru
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 *  USA
 *
 *  $Id: $
 */

#include "common.h"
#include "stdinc.h"
#include "fileio.h"
#include "memory.h"
#include "client.h"
#include "translate.h"
#include "s_user.h"
#include "s_log.h"
#include "s_conf.h"

static struct Codepage *codepages = NULL;

struct Codepage *translate_codepage_add(const char *charset, const char *file)
{
  struct Codepage *cp;
  int fd = 0, cnt = 0;

  if (charset == NULL)
    return NULL;

  if ((cp = translate_codepage_find_byname(charset)))
  {
    return cp;
  }

  ilog(L_WARN, "Load codepage %s from %s", charset, file);

  if (codepages)
  {
    cp = codepages;
    while (cp->next)
    {
      cp = cp->next;
    }
    cp->next = (struct Codepage*) MyMalloc(sizeof(struct Codepage));
    cp->next->prev = cp;
    cp->next->next = NULL;
    cp = cp->next;
  }
  else
  {
    codepages = cp = (struct Codepage*) MyMalloc(sizeof(struct Codepage));
    cp->prev = cp->next = NULL;
  }

  strlcpy(cp->name, charset, 20);

  if ((fd = file_open(file, O_RDONLY, 0)) == -1)
  {
    if (cp->prev)
      cp->prev->next = NULL;
    if ((cp->prev == NULL) && (cp->next == NULL))
      codepages = NULL;
    MyFree(cp);
    return NULL;
  }
  cnt += read(fd, cp->to, 256);
  cnt += read(fd, cp->from, 256);
  file_close(fd);
  if (cnt == 512)
  {
    return cp;
  }
  else
  {
    if (cp->prev)
      cp->prev->next = NULL;
    MyFree(cp);
    return NULL;
  }
}

struct Codepage *translate_codepage_find_byname(const char *charset)
{
  struct Codepage *cp = codepages;

  while (cp)
  {
    if (!strcasecmp(cp->name, charset))
      break;

    cp = cp->next;
  }
  
  return cp;
}
